/*
 *  
 *  $Id: controladorcomandos.cpp 3526 2011-03-16 19:56:19Z carlos $
 *  Ginkgo CADx Project
 *
 *  Copyright 2008-10 MetaEmotion S.L. All rights reserved.
 *  http://ginkgo-cadx.com
 *
 *  This file is licensed under LGPL v3 license.
 *  See License.txt for details
 *
 *
 */
//#define _GINKGO_TRACE

#include <api/globals.h>
#include <api/icomando.h>

#include <main/controllers/controladorlog.h>
#include <main/controllers/controladoreventos.h>
#include <eventos/eventosginkgo.h>

#include <wx/wx.h>
#include <wx/thread.h>

#include "controladorcomandos.h"
#include <main/gui/mainwindow/ventanaprincipal.h>

//#undef GTRACE
//#define GTRACE(expr) std::cout << expr << std::endl

#if defined (_WINDOWS)
#define MS_VC_EXCEPTION 0x406D1388

#pragma pack(push,8)
typedef struct tagTHREADNAME_INFO
{
   DWORD dwType; // Must be 0x1000.
   LPCSTR szName; // Pointer to name (in user addr space).
   DWORD dwThreadID; // Thread ID (-1=caller thread).
   DWORD dwFlags; // Reserved for future use, must be zero.
} THREADNAME_INFO;
#pragma pack(pop)

void GNC::GCS::Threading::SetThreadName( long threadID, const std::string& threadName)
{
   //Sleep(10);
   THREADNAME_INFO info;
   info.dwType = 0x1000;
   info.szName = threadName.c_str();
   info.dwThreadID = threadID;
   info.dwFlags = 0;

   __try
   {
      RaiseException( MS_VC_EXCEPTION, 0, sizeof(info)/sizeof(ULONG_PTR), (ULONG_PTR*)&info );
   }
   __except(EXCEPTION_EXECUTE_HANDLER)
   {
   }
}
#elif defined(__WXOSX__)
#include <pthread.h>
#include <sys/signal.h>

void GNC::GCS::Threading::SetThreadName( long /*threadID*/, const std::string& threadName)
{
	//pthread_setname_np(threadName.c_str());
}

void FooSignalHandler(int sig)
{
	GTRACE("SIGUSR1 recibido");
}
#else
#include <pthread.h>
#include <sys/signal.h>

void GNC::GCS::Threading::SetThreadName( long threadID, const std::string& threadName)
{
}

void FooSignalHandler(int sig)
{
	GTRACE("SIGUSR1 recibido");
}

#endif
GNC::LanzadorComandos::LanzadorComandos(GNC::GCS::IComando* pComando)
{
	//std::cout << "LanzadorComandos::LanzadorComandos()" << this << std::endl;
	m_idThreadGinkgo = 0;
	m_pComando = pComando;
	m_Abortar = false;
	//this->Create();
	m_pComando->SetNotificadorProgreso(this);
	//NotificarProgreso(0.0f, m_pComando->GetName());
}

GNC::LanzadorComandos::~LanzadorComandos()
{
	GTRACE("<< LanzadorComandos::~LanzadorComandos() " << m_idThreadGinkgo);
}

void* GNC::LanzadorComandos::Entry()
{
	GTRACE(">> LanzadorComandos::Entry() " << m_idThreadGinkgo << " " << m_pComando->GetName());
	this->IniciarUnlocker();

	GNC::GCS::Threading::SetThreadName( GetId(), m_pComando->GetName());

	if (m_pComando != NULL && !m_pComando->EstaAbortado() && m_Dependencias.Size(GLOC()) > 0) {
		GTRACE(">> LanzadorComandos::Entry() " << m_idThreadGinkgo << " " << m_pComando->GetName() << ") espera " << m_Dependencias.Size(GLOC()) << " tareas");
		while (m_Dependencias.NotEmpty(GLOC())) {
			if (!m_Dependencias.Wait(1000, GLOC())) {
				//std::cout << "Timeout de espera para " << m_pComando->GetName() << "(" << m_idThreadGinkgo << ") " << m_Dependencias.Size(GLOC()) << " pendientes" << std::endl;
				m_Dependencias.TerminarPendientes(GLOC());
			}
		}
		GTRACE(">> LanzadorComandos::Entry() " << m_idThreadGinkgo << " " << m_pComando->GetName() << " espera terminada");
	}

	wxWindow* w = GNC::GCS::ControladorComandos::Instance()->GetVentanaRegistrada();

	bool iniciado = false;

	if (m_pComando != NULL && !m_pComando->EstaAbortado()) {

		if (w != NULL) {
			iniciado = true;
			EventoProgreso ce(EventoProgreso::ComandoIniciado, m_idThreadGinkgo);
			w->AddPendingEvent(ce);
		}

		try {
			m_pComando->Execute();
		}
		catch (GNC::GCS::ControladorComandosException& ) {
			//comando abortado
		}
		catch (const std::bad_alloc& ) {
			//comando ha cascado por un bad alloc
		}
		catch (const std::exception& ) {
			//comando abortado
		}
		catch (...) {
			LOG_ERROR("LanzadorComandos", "Error interno ejecutando el comando con id = " << m_pComando->GetId());
		}

	}




	GTRACE("<< LanzadorComandos::Entry(): "  << m_idThreadGinkgo << " " << m_pComando->GetName());
	return NULL;
}

bool GNC::LanzadorComandos::RecalcularDependencias()
{
	//revisamos la lista de "aborta si" ya que en ese caso no arrancamos

	if(m_Abortar){
		return false;
	}

	//GTRACE(">> comprobando el ''AbortaSi'' y el ''CancelaA'' : " << m_idThreadGinkgo);
	GNC::GCS::ControladorComandos::MapaComandos* mapaComandosLanzados = &(GNC::GCS::ControladorComandos::Instance()->m_ComandosLanzados);
	for(GNC::GCS::ControladorComandos::MapaComandos::iterator it = mapaComandosLanzados->begin(); it!= mapaComandosLanzados->end(); ++it){
		LanzadorComandos* pLanzador = (*it).second;
		if (pLanzador == this) {
			continue;
		}

		for(GNC::GCS::TipoListaIdComandos::iterator it1 = m_pComando->m_Conflictos.begin(); it1 != m_pComando->m_Conflictos.end(); ++it1){
			if(pLanzador->GetComando()->GetId() == (*it1) && pLanzador->m_idThreadGinkgo != m_idThreadGinkgo){
				GTRACE("\tNo se registra espera: " << m_pComando->GetName() << "(" << m_idThreadGinkgo << ") aborta_si " << pLanzador->m_pComando->GetName() << "(" << pLanzador->m_idThreadGinkgo << ") = " << this);
				//GTRACE(">> encontrado un comando de ''AbortaSi'' => NO SE EJECUTA " << m_idThreadGinkgo);
				return false;
			}
		}
	}

	for(GNC::GCS::ControladorComandos::MapaComandos::iterator it = mapaComandosLanzados->begin(); it!= mapaComandosLanzados->end(); ++it){
		LanzadorComandos* pLanzador = (*it).second;
		if (pLanzador == this) {
			continue;
		}

		for(GNC::GCS::TipoListaIdComandos::iterator it1 = m_pComando->m_Reemplazos.begin(); it1 != m_pComando->m_Reemplazos.end(); ++it1){
			if(pLanzador->GetComando()->GetId() == (*it1) && pLanzador->m_idThreadGinkgo != m_idThreadGinkgo){
				//GTRACE(">> encontrado un comando de ''CancelaA'' => El hilo " << m_idThreadGinkgo<< " cancelara a "<<pLanzador->m_idThreadGinkgo);
				if (pLanzador != NULL) { //
					m_Dependencias.RegistrarEspera(pLanzador, GLOC());
					GTRACE("\tRegistrada espera: " << m_pComando->GetName() << "(" << m_idThreadGinkgo << ") cancela_a " << pLanzador->m_pComando->GetName() << "(" << pLanzador->m_idThreadGinkgo << ") = " << this);
					pLanzador->Terminar();
				}
				else {
					std::cerr << "Detectada inconsistencia en el controlador de comandos: Comando lanzado desreferenciado. Flujo de comando perdido." << std::endl;
				}
			}
		}

		for(GNC::GCS::TipoListaIdComandos::iterator it1 = m_pComando->m_Dependencias.begin(); it1 != m_pComando->m_Dependencias.end(); ++it1){
			if(pLanzador != NULL && pLanzador->GetComando()->GetId() == (*it1) && pLanzador->m_idThreadGinkgo != m_idThreadGinkgo){
				//GTRACE(">> encontrado un comando de ''Espera a'' => El hilo " << m_idThreadGinkgo<< " esprerara a "<<pLanzador->m_idThreadGinkgo);
				//nos aseguramos de que el no esta esperando por nosostros... en tal caso habría un bloqueo mutuo!!!
				//solo se permite en caso de que los dos comandos tengan el mismo id por ejemplo comandos que se tienen que ejecutar en serie,
				//en tal caso solo le esperamos si su id es menor que el nuestro
				if(pLanzador->GetComando()->GetId() != this->GetComando()->GetId() ||
					(pLanzador->GetComando()->GetId() == this->GetComando()->GetId() && m_idThreadGinkgo > pLanzador->m_idThreadGinkgo))
				{
					m_Dependencias.RegistrarEspera(pLanzador, GLOC());
					GTRACE("\tRegistrada espera: " << m_pComando->GetName() << "(" << m_idThreadGinkgo << ") espera_a " << pLanzador->m_pComando->GetName() << "(" << pLanzador->m_idThreadGinkgo << ") = " << this);
				}
			}
		}
	}
	return true;
}

void GNC::LanzadorComandos::OnExit()
{
	GTRACE(/*"LanzadorComandos",*/ ">> Comando Terminado");
	{
		GNC::GCS::ILocker pLocker(GNC::GCS::ControladorComandos::Instance(), GLOC());

		GTRACE(">> LanzadorComandos::OnExit() " << m_idThreadGinkgo << " " << m_pComando->GetName());

		//region "Region sincrona con ControladorComandos"
		m_pComando->SetNotificadorProgreso(NULL);

		GNC::GCS::ControladorComandos::MapaComandos::iterator it = GNC::GCS::ControladorComandos::Instance()->m_ComandosLanzados.find(m_idThreadGinkgo);
		if (it != GNC::GCS::ControladorComandos::Instance()->m_ComandosLanzados.end()) {
			GNC::GCS::ControladorComandos::Instance()->m_ComandosLanzados.erase(it);
		}

		GTRACE("  LanzadorComandos::OnExit(): Notificando pendientes. threadId = " << m_idThreadGinkgo);

		//avisa a los que estan esperando

		wxWindow* w = GNC::GCS::ControladorComandos::Instance()->GetVentanaRegistrada();

		if (w != NULL) {
			GNC::GCS::ControladorComandos::Instance()->m_ComandosTerminados[m_idThreadGinkgo] = m_pComando;
			GTRACE(/*"LanzadorComandos::OnExit()", */"Propagando evento de finalizacion a la ventana principal");
			EventoProgreso ce(EventoProgreso::ComandoFinalizado, m_idThreadGinkgo);
			w->AddPendingEvent(ce);
		}
		else {
			LOG_WARN("LanzadorComandos::OnExit()", "Error: No se invocara el evento de finalizacion del comando. El sistema puede quedar bloqueado o inestable");
			if (m_pComando != NULL) {
				m_pComando->Abort(); // Notificamos al comando de que va a abortar.
				delete m_pComando;
				m_pComando = NULL;
			}
		}
	}

	GTRACE(">> LanzadorComandos::OnExit() " << m_idThreadGinkgo << " Notificación a " << m_ListaNotificacion.size() << " tareas que esperan");


	Signal(GLOC());

	//LOG_TRACE("LanzadorComandos", "<< Comando Terminado");

	//endregion

	GTRACE("<< LanzadorComandos::OnExit() " << m_idThreadGinkgo);
}

void GNC::LanzadorComandos::IniciarUnlocker()
{
	#if defined(_WINDOWS)

	#else
	/*
	struct sigaction action;
	memset(&action, 0, sizeof(action));
	sigemptyset(&action.sa_mask);
	action.sa_flags = 0;
	action.sa_handler = FooSignalHandler;
	sigaction(SIGUSR2, &action, NULL);
	*/
	#endif
}

void GNC::LanzadorComandos::DoTerminar()
{
	this->m_Abortar = true;
	if (m_pComando != NULL) {
		//std::cout << "LanzadorComandos::DoTerminar(): Abortando comando:" << std::endl;
		m_pComando->Abort();
	}
	#if defined(_WINDOWS)

	#else
	//pthread_kill((pthread_t)this->GetId(), SIGUSR2);

	#endif
}

bool GNC::LanzadorComandos::NotificarProgreso(float progresoNormalizado, const std::string& texto)
{
	//GNC::GCS::ILocker pLocker(this, GLOC());
	if(m_Abortar){
		//std::cout << "Comando abortado: No se notificara el progreso" << std::endl;
		return false;
	}
	//GTRACE(">> LanzadorComandos::NotificarProgreso(): " << m_idThreadGinkgo);
	LockProgreso(GLOC());
	SetProgresoNormalizado(std::max<float>(0, std::min<float>(progresoNormalizado,1.0f)));
	SetTextoProgreso(texto);
	UnLockProgreso(GLOC());


	wxWindow* w = GNC::GCS::ControladorComandos::Instance()->GetVentanaRegistrada();
	if(w!=NULL) {
		EventoProgreso ce(EventoProgreso::ComandoEjecutando, m_idThreadGinkgo);
		//std::cout << "AddPendingEvent EventoProgreso::ComandoEjecutando " << m_idThreadGinkgo << std::endl;
		w->GetEventHandler()->AddPendingEvent(ce);
	}
	//GTRACE("<< LanzadorComandos::NotificarProgreso(): " << m_idThreadGinkgo);
	return true;

}

//----------------------------------------------------------------------------------------------------
//----------------------------------------------------------------------------------------------------

//region Controlador de comandos

GNC::GCS::ControladorComandos* GNC::GCS::ControladorComandos::m_psInstancia = NULL;

GNC::GCS::ControladorComandos::ControladorComandos()
{
	m_Destroying = false;
	m_pWin = NULL;
	m_pProgreso = NULL;
	m_idThreads = 1;
}

GNC::GCS::ControladorComandos::~ControladorComandos()
{
	GTRACE(">> ControladorComandos::~ControladorComandos() ");
	AbortarComandosDeOwner(NULL);
	GTRACE("<< ControladorComandos::~ControladorComandos() ");
}

GNC::GCS::ControladorComandos* GNC::GCS::ControladorComandos::Instance()
{
	if (m_psInstancia == NULL) {
		m_psInstancia = new ControladorComandos();
	}
	return m_psInstancia;
}

void GNC::GCS::ControladorComandos::FreeInstance()
{
	if (m_psInstancia != NULL) {
		delete m_psInstancia;
		m_psInstancia = NULL;
	}

}

void GNC::GCS::ControladorComandos::Process(GNC::GCS::IComando * cmd)
{
	cmd->Execute();
	cmd->Update();
	delete cmd;
}

void GNC::GCS::ControladorComandos::ProcessAsync(std::string str, GNC::GCS::IComando* cmd, void * owner)
{
	cmd->SetOwner(owner);
	cmd->SetNotificadorProgreso(NULL);
	LanzadorComandos* thread = new LanzadorComandos(cmd);
	//thread->SetPriority(WXTHREAD_MAX_PRIORITY);

	GNC::GCS::ILocker locker(this, GLOC());

	long threadId = m_idThreads++;
	thread->m_idThreadGinkgo = threadId;

	int res = -1;
	int creado = -1;
	try {
		if (thread->RecalcularDependencias()) {
			creado = thread->Create();
			res = thread->Run();
		}
		else {
			delete cmd;
			if(creado == wxTHREAD_NO_ERROR)
			{
				thread->Delete();
			} else {
				delete thread;
			}
			thread = NULL;
		}
	}
	catch (GNC::GCS::ControladorComandosException& ex) {
		LOG_ERROR("ControladorComandos", "Error al lanzar el comando: " << ex);
		delete cmd;
		if(creado == wxTHREAD_NO_ERROR)
		{
			thread->Delete();
		} else {
			delete thread;
		}
		thread = NULL;
	}
	catch (...) {
		LOG_ERROR("ControladorComandos", "Error interno al lanzar el comando");
		delete cmd;
		if(creado == wxTHREAD_NO_ERROR)
		{
			thread->Delete();
		}
		else {
			delete thread;
		}
		thread = NULL;
	}
	if ( res == wxTHREAD_NO_ERROR && creado == wxTHREAD_NO_ERROR)
	{
		m_ComandosLanzados[threadId] = thread;
		MapaOwners::iterator it = m_MapaOwners.find(owner);
		if (it != m_MapaOwners.end())
		{
			(*it).second.push_back(threadId);
		}
		else
		{
			ListaHilos l;
			l.push_back(threadId);
			m_MapaOwners[owner] = l;
		}
	}
}

//este metodo es un poco malo porque la busqueda iterada...
void GNC::GCS::ControladorComandos::AbortarComando(IComando* pComando, bool sincrono)
{
	GNC::GCS::WaitQueue wqueue;
	GNC::GCS::ILocker locker(this, GLOC());
	GTRACE(">> GNC::GCS::ControladorComandos::AbortarComando(pComando) ");
	// Recorremos la lista de threads_ids de la vista
	for(MapaComandos::iterator it = m_ComandosLanzados.begin(); it!= m_ComandosLanzados.end(); ++it){
		LanzadorComandos* pLanzador = (*it).second;
		if(pLanzador == NULL || (*it).second->GetComando() != pComando){
			continue;
		}
		GTRACE("\t\t Localizado: " << pLanzador->m_pComando->GetName() << "(" << pLanzador->m_idThreadGinkgo << ")");
		if (sincrono ) {
			wqueue.RegistrarEspera(pLanzador, GLOC());
			std::cerr << "Registrando espera para tarea: ptr = " << pLanzador << std::endl;
		}
		pLanzador->Terminar();
	}
	GTRACE("<< ControladorComandos::AbortarComandos(pComando)");
}

void GNC::GCS::ControladorComandos::AbortarComando(long threadId, bool sincrono)
{
	GNC::GCS::WaitQueue wqueue;

	//---------------------------------------------------------------------
	// REGION CRITICA
	//---------------------------------------------------------------------
	{ // BEGIN Modificación del estado interno del controlador de comandos.
		GNC::GCS::ILocker locker(this, GLOC());
		GTRACE(">> ControladorComandos::AbortarComando(long) : CS");
		// Recorremos la lista de threads_ids de la vista
		MapaComandos::iterator it = m_ComandosLanzados.find(threadId);
		if (it != m_ComandosLanzados.end()) {
			LanzadorComandos* pLanzador = (*it).second;
			if (pLanzador != NULL) { //
				if (sincrono ) {
					wqueue.RegistrarEspera(pLanzador, GLOC());
					std::cerr << "Registrando espera para tarea: threadId = " << (unsigned long) threadId << " ptr = " << pLanzador << std::endl;
				}
				pLanzador->Terminar();
			} else {
				std::cerr << "Detectada inconsistencia en el controlador de comandos: Comando lanzado desreferenciado. Flujo de comando perdido." << std::endl;
			}
		} else { // Buscamos el id en la lista de comandos terminados
			MapaComandosTerminados::iterator it3 = m_ComandosTerminados.find(threadId);
			if (it3 != m_ComandosTerminados.end()) {
				IComando* pComando = (*it3).second;
				if (pComando != NULL) {
					std::cerr << "Anulando comando terminado: threadId = " << (unsigned long) threadId << std::endl;
					pComando->Abort();
					delete pComando;
				} else {
					std::cerr << "Detectada inconsistencia en el controlador de comandos: Comando terminado nulo." << std::endl;
				}
				m_ComandosTerminados.erase(it3);
			} else {
				std::cerr << "Detectada inconsistencia en el controlador de comandos: Comando desreferenciado. Flujo de comando perdido." << std::endl;
			}
		}

		GTRACE("<< ControladorComandos::AbortarComandos(long) : CS");
	}
	// END Modificación del estado interno del controlador de comandos.
	//---------------------------------------------------------------------
	//---------------------------------------------------------------------
	GTRACE("GNC::GCS::ControladorComandos::AbortarComando( " << threadId << ") : Esperando " << wqueue.Size(GLOC()) << " tareas");
	while (wqueue.NotEmpty()) {
		if (!wqueue.Wait(500, GLOC())) {
			wqueue.TerminarPendientes(GLOC() );
		}
	}
	GTRACE("GNC::GCS::ControladorComandos::AbortarComando( " << threadId << ") : Espera terminada");

}

void GNC::GCS::ControladorComandos::AbortarComandosDeOwner(void * owner)
{

	GNC::GCS::WaitQueue wqueue;

	{
		GNC::GCS::ILocker locker(this, GLOC());
		GTRACE(">> ControladorComandos::AbortarComandosDeOwner(owner) : CS");
		// Recorremos la lista de threads_ids de la vista
		MapaOwners::iterator it1 = m_MapaOwners.find(owner);
		MapaComandos::iterator itcl;
		if (it1 != m_MapaOwners.end()) {
			for (ListaHilos::iterator it2 = (*it1).second.begin(); it2 != (*it1).second.end(); it2++) {
				long threadId = (*it2);
				// Buscamos el id en la lista de comandos lanzados
				itcl = m_ComandosLanzados.find(threadId);
				if (itcl != m_ComandosLanzados.end()) { // El comando esta lanzado
					LanzadorComandos* pLanzador = (*itcl).second;
					if (pLanzador != NULL) { //

						wqueue.RegistrarEspera(pLanzador, GLOC());
						pLanzador->Terminar();

						std::cerr << "Registrando espera para tarea: threadId = " << (unsigned long) threadId << " ptr = " << pLanzador << std::endl;
					} else {
						std::cerr << "Detectada inconsistencia en el controlador de comandos: Comando lanzado desreferenciado. Flujo de comando perdido." << std::endl;
					}
				} else { // Buscamos el id en la lista de comandos terminados
					MapaComandosTerminados::iterator it3 = m_ComandosTerminados.find(threadId);
					if (it3 != m_ComandosTerminados.end()) {
						IComando* pComando = (*it3).second;
						if (pComando != NULL) {
							std::cerr << "Anulando comando terminado: threadId = " << (unsigned long) threadId << std::endl;
							pComando->Abort();
							delete pComando;
						} else {
							std::cerr << "Detectada inconsistencia en el controlador de comandos: Comando terminado nulo." << std::endl;
						}
						m_ComandosTerminados.erase(it3);
					} else {
						std::cerr << "Detectada inconsistencia en el controlador de comandos: Comando desreferenciado. Flujo de comando perdido." << std::endl;
					}
				}
			}
			m_MapaOwners.erase(it1);
		}
		GTRACE("<< ControladorComandos::AbortarComandosDeOwner(owner) : CS");
	}

	GTRACE("GNC::GCS::ControladorComandos::AbortarComandosDeOwner( " << owner << ") : Esperando " << wqueue.Size(GLOC()) << " tareas");
	while (wqueue.NotEmpty(GLOC())) {
		if (!wqueue.Wait(500, GLOC())) {
			wqueue.TerminarPendientes(GLOC());
		}
	}
	GTRACE("GNC::GCS::ControladorComandos::AbortarComandosDeOwner( " << owner << ") : Espera terminada");
}

void GNC::GCS::ControladorComandos::AbortarComandosDeOwnerAsincrono(void * owner)
{
	GNC::GCS::ILocker locker(this, GLOC());
	GTRACE(">> ControladorComandos::AbortarComandosDeOwner(owner) : CS");
	// Recorremos la lista de threads_ids de la vista
	MapaOwners::iterator it1 = m_MapaOwners.find(owner);
	if (it1 != m_MapaOwners.end()) {
		for (ListaHilos::iterator it2 = (*it1).second.begin(); it2 != (*it1).second.end(); it2++) {
			long threadId = (*it2);
			// Buscamos el id en la lista de comandos lanzados
			MapaComandos::iterator it = m_ComandosLanzados.find(threadId);
			if (it != m_ComandosLanzados.end()) { // El comando esta lanzado
				LanzadorComandos* pLanzador = (*it).second;
				if (pLanzador != NULL) { //
					pLanzador->Terminar();
				} else {
					std::cerr << "Detectada inconsistencia en el controlador de comandos: Comando lanzado desreferenciado. Flujo de comando perdido." << std::endl;
				}
			} else { // Buscamos el id en la lista de comandos terminados
				MapaComandosTerminados::iterator it3 = m_ComandosTerminados.find(threadId);
				if (it3 != m_ComandosTerminados.end()) {
					IComando* pComando = (*it3).second;
					if (pComando != NULL) {
						std::cerr << "Anulando comando terminado: threadId = " << (unsigned long) threadId << std::endl;
						pComando->Abort();
						delete pComando;
					} else {
						std::cerr << "Detectada inconsistencia en el controlador de comandos: Comando terminado nulo." << std::endl;
					}
					m_ComandosTerminados.erase(it3);
				} else {
					std::cerr << "Detectada inconsistencia en el controlador de comandos: Comando desreferenciado. Flujo de comando perdido." << std::endl;
				}
			}
		}
		m_MapaOwners.erase(it1);
	}
	GTRACE("GNC::GCS::ControladorComandos::AbortarComandosDeOwnerAsincrono( " << owner << ")");
	GTRACE("<< ControladorComandos::AbortarComandosDeOwner(owner) : CS");
}

void GNC::GCS::ControladorComandos::OnComandoLanzado(long threadId)
{

	GNC::GCS::ILocker locker(this, GLOC());

	MapaComandos::iterator it = m_ComandosLanzados.find(threadId);
	if (it != m_ComandosLanzados.end()) {
		LanzadorComandos* pLanzador = (*it).second;
		pLanzador->m_pComando->SetNotificadorProgreso(pLanzador);
		if (m_pProgreso) {
			m_pProgreso->InsertarTarea(threadId, pLanzador->m_pComando->GetName());
		}
		GNC::GCS::ControladorEventos::Instance()->ProcesarEvento(new GNC::GCS::Eventos::EventoProgresoComando(pLanzador->m_pComando, GNC::GCS::Eventos::EventoProgresoComando::TEP_Iniciado, 0.0f, pLanzador->m_pComando->GetName()));
	}
	else {
		MapaComandosTerminados::iterator it = m_ComandosTerminados.find(threadId);
		if (it != m_ComandosTerminados.end()) {
			IComando* pComando = (*it).second;
			if (m_pProgreso) {
				m_pProgreso->InsertarTarea(threadId, pComando->GetName());
			}
			GNC::GCS::ControladorEventos::Instance()->ProcesarEvento(new GNC::GCS::Eventos::EventoProgresoComando(pComando, GNC::GCS::Eventos::EventoProgresoComando::TEP_Iniciado, 1.0f, pComando->GetName()));
		}
		else
		{
			LOG_WARN("ControladorComandos", "Error al iniciar comunicacion de progreso del comando. Thread Id no encontrado.");
			//std::cout << "OnComandoLanzado: Thread Id not found: " << threadId << std::endl;
		}
	}
}

void GNC::GCS::ControladorComandos::OnComandoProgreso(long threadId)
{
	GNC::GCS::ILocker locker(this, GLOC());

	MapaComandos::iterator it = m_ComandosLanzados.find(threadId);
	if (it != m_ComandosLanzados.end()) {
		LanzadorComandos* pLanzador = (*it).second;
		pLanzador->LockProgreso(GLOC());
		float progreso = pLanzador->GetProgresoNormalizado();
		std::string texto = pLanzador->GetTextoProgreso();
		pLanzador->UnLockProgreso(GLOC());
		m_pProgreso->SetProgresoTarea(threadId, progreso, texto);
		GNC::GCS::ControladorEventos::Instance()->ProcesarEvento(new GNC::GCS::Eventos::EventoProgresoComando(pLanzador->m_pComando, GNC::GCS::Eventos::EventoProgresoComando::TEP_Progreso, progreso, texto));
	}
	else {
		//std::cout << "OnComandoProgreso: Thread Id not found: " << threadId << std::endl;
	}

}

void GNC::GCS::ControladorComandos::OnComandoFinalizado(long threadId, bool lock)
{
	IComando* pComando = NULL;

	bool actualizar=false;

	GNC::GCS::ILocker* pLocker = NULL;

	//LOG_TRACE("ControladorComandos", "OnComandoFinalizado(" << threadId << ")");

	{
		if (lock) {
			pLocker = new GNC::GCS::ILocker(this, GLOC());
		}
		if(m_pProgreso != NULL){
			m_pProgreso->EliminarTarea(threadId);
		}

		MapaComandosTerminados::iterator it1 = m_ComandosTerminados.find(threadId);
		if (it1 != m_ComandosTerminados.end()) {
			actualizar = true;
			pComando = (*it1).second;
			m_ComandosTerminados.erase(it1);
		}
		else {
			//LOG_TRACE("ControladorComandos", "Id thread no encontrado en finalizados");
		}


		// TODO: Añadir un diccionario inverso o usar un set
		for (MapaOwners::iterator it2 = m_MapaOwners.begin(); it2 != m_MapaOwners.end(); it2++) {
			ListaHilos::iterator it3 = (*it2).second.begin();
			bool found = false;
			while (!found && it3 != (*it2).second.end()) {
				if ((*it3) == threadId) {
					found = true;
				}
				else {
					it3++;
				}
			}
			if (found) {
				GTRACE("ControladorComandos::OnComandoFinalizado(): threadId " << (unsigned long) threadId << " eliminado de la lista de owners");
				(*it2).second.erase(it3);
			}
		}

		if (pLocker != NULL)
		{
			delete pLocker;
			pLocker = NULL;
		}
	}

	if (pComando != NULL)
	{
		if (actualizar && !pComando->EstaAbortado()) {
			pComando->Update();
		}
		else {
			LOG_DEBUG("ControladorComandos" , "IComando::Update no invocado");
		}
		//LOG_TRACE("ControladorComandos", "Procesando evento de comando finalizado");
		GNC::GCS::ControladorEventos::Instance()->ProcesarEvento(new GNC::GCS::Eventos::EventoProgresoComando(pComando, GNC::GCS::Eventos::EventoProgresoComando::TEP_Finalizado));
		GTRACE(/*"ControladorComandos", */"Eliminando comando...");
		delete pComando;
		GTRACE(/*"ControladorComandos", */"Comando eliminado");
	}
	else {
		LOG_WARN("ControladorComandos", "Comando nulo. no se propagara el evento de fin de comando ni se actualizara");
		//std::cout << "ControladorComandos>> pComando = NULL!!!!" << std::endl;
	}

}

GNC::GCS::ControladorComandos::ListaNombreComandos GNC::GCS::ControladorComandos::GetComandosActivos()
{
	ListaNombreComandos lista;
	GNC::GCS::ILocker locker(this, GLOC());
	for(MapaComandos::iterator it = m_ComandosLanzados.begin(); it!=m_ComandosLanzados.end(); it++) {
		std::stringstream strstream;
		strstream << (*it).second->m_pComando->GetName() << ": " << (*it).second->GetTextoProgreso();
		lista.push_back( strstream.str());
	}
	return lista;
}

//endregion
