/*
*
*  $Id: xmlrpccontroller.cpp 3712 2011-04-15 11:29:44Z carlos $
*  Ginkgo CADx Project
*
*  Copyright 2008-10 MetaEmotion S.L. All rights reserved.
*  http://ginkgo-cadx.com
*
*  This file is licensed under LGPL v3 license.
*  See License.txt for details
*
*
*/

#include <wx/file.h>
#include <wx/filename.h>
#include <wx/dir.h>
#include <wx/config.h>
#include <wx/wxsqlite3/wxsqlite3.h>

#include "xmlrpccontroller.h"
#include <api/globals.h>
#include <api/internacionalizacion.h>
#define LOGGER "XMLRPCController"
#include "controladorlog.h"
#include "../entorno.h"
#include <api/icomando.h>
#include <main/gui/mainwindow/ventanaprincipal.h>
#include <main/gui/history/panelhistorial2.h>
#include <main/gui/history/ipanelhistorial.h>
#include <main/gui/import/wxwizardimportacionginkgo.h>
#include "controladorcomandos.h"
#include "controladoreventos.h"
#include <eventos/eventosginkgo.h>
#include "controladorhl7.h"
#include <xmlrpc/XmlRpc.h>
#include <commands/comandopacs.h>
#include <api/icontextoestudio.h>
#include <set>

#define IDC_RPC 115

//log handler...
namespace XmlRpc {
	//------------------------------------------------------------------------------------------------------+
	//TODO: It's not working, when you activate log redirection to ControladorLog client ends unexpectly
	class GinkgoLogHandler: public XmlRpc::XmlRpcLogHandler
	{
	public:
		GinkgoLogHandler() {}
		~GinkgoLogHandler() {}

		virtual void log(int level, const char* msg)
		{
			GNC::GCS::ControladorLog::Instance()->Log(msg, (GNC::GCS::IControladorLog::LogLevel)level);
		}
	};
	//------------------------------------------XML_RPC METHODS--------------------------------------------------
	//"CloseAllWindows", closes all opened windows.
	//ClosesAllWindowsCommand, it's necessary to launch a command because closing method has to be called by interface thread
	class ClosesAllWindowsCommand : public GNC::GCS::IComando {
	public:
		ClosesAllWindowsCommand() : IComando(NULL)
		{
			SetId(IDC_RPC);
		}

		virtual void Execute() {}

		virtual void Update()
		{
			if (GNC::Entorno::Instance()->GetVentanaPrincipal()->CerrarTodosLosTabsYPanelGrids())
			{
				GNC::Entorno::Instance()->GetVentanaPrincipal()->ClosesAllUndocked();
			}
		}
	};

	//CloseAllWindows this is the rpc command, it launches Ginkgo Command
	class CloseAllWindows : public XmlRpcServerMethod
	{
	public:
		CloseAllWindows(XmlRpcServer* s) : XmlRpcServerMethod("CloseAllWindows", s) {}

		void execute(XmlRpcValue& /*params*/, XmlRpcValue& result)
		{
			GNC::GCS::ControladorComandos::Instance()->ProcessAsync(_Std("Closing windows..."), new ClosesAllWindowsCommand(), NULL);
			result["error"] = 0;
		}

		std::string help() { return std::string("Closes all opened windows"); }

	};
	//---------------------------------------------------------------------------------------------------
	// Close2DViewerWithSeriesUID and Close2DViewerWithStudyUID, closes view with tab uid as specified
	// ClosesViewCommand, it's necessary to launch a command because closing method has to be called by interface thread
	class ClosesViewCommand : public GNC::GCS::IComando {
	public:
		ClosesViewCommand(const std::string& uid, bool series = true) : IComando(NULL)
		{
			m_series = series;
			m_uid = uid;
			SetId(IDC_RPC);
		}

		virtual void Execute() {}

		virtual void Update()
		{
			GNC::GUI::PanelHistorial2* pPanel = GNC::Entorno::Instance()->GetVentanaPrincipal()->GetPanelHistorial();
			GNC::GCS::IVista* pVista;
			if (m_series) {
				pVista = pPanel->GetVistaFromSeriesUID(m_uid);
			} else {
				pVista = pPanel->GetVistaFromStudyUID(m_uid);
			}
			if (pVista != NULL) {
				GNC::Entorno::Instance()->GetVentanaPrincipal()->CerrarVista(pVista);
			}
		}
		std::string m_uid;
		bool m_series;
	};

	//Close2DViewerWithSeriesUID this is the rpc command, it parses params and launches Ginkgo Command
	class Close2DViewerWithSeriesUID : public XmlRpcServerMethod
	{
	public:
		Close2DViewerWithSeriesUID(XmlRpcServer* s) : XmlRpcServerMethod("Close2DViewerWithSeriesUID", s) {}

		void execute(XmlRpcValue& params, XmlRpcValue& result)
		{
			std::string uidSeries = std::string(params[0]["uid"]);
			if (uidSeries != "") {
				GNC::GCS::ControladorComandos::Instance()->ProcessAsync(_Std("Closing windows..."), new ClosesViewCommand(uidSeries), NULL);
			}
			result["error"] = 0;
		}

		std::string help() { return std::string("Closes series"); }

	};

	//Close2DViewerWithStudyUID this is the rpc command, it parses params and launches Ginkgo Command
	class Close2DViewerWithStudyUID : public XmlRpcServerMethod
	{
	public:
		Close2DViewerWithStudyUID(XmlRpcServer* s) : XmlRpcServerMethod("Close2DViewerWithStudyUID", s) {}

		void execute(XmlRpcValue& params, XmlRpcValue& result)
		{
			std::string uidStudy = std::string(params[0]["uid"]);
			if (uidStudy != "") {
				GNC::GCS::ControladorComandos::Instance()->ProcessAsync(_Std("Closing windows..."), new ClosesViewCommand(uidStudy, false), NULL);
			}
			result["error"] = 0;
		}

		std::string help() { return std::string("Closes study"); }

	};
	//---------------------------------------------------------------------------------------------------
	//OpenDB changes Ginkgo's dicomdir path
	//ChangeDicomDirCommand command it's necessary because interface will be refreshed
	class ChangeDicomDirCommand : public GNC::GCS::IComando {
	public:
		ChangeDicomDirCommand() : IComando(NULL)
		{
			SetId(IDC_RPC);
		}

		virtual void Execute() {}

		virtual void Update()
		{
			GNC::Entorno::Instance()->RefrescarDicomDir();
			GNC::GCS::ControladorEventos::Instance()->ProcesarEvento(new GNC::GUI::Eventos::EventoRecargarHistorial());
		}
	};

	//OpenDB parses params and if it's necessary lauches Change dicomdir command
	class OpenDB : public XmlRpcServerMethod
	{
	public:
		OpenDB(XmlRpcServer* s) : XmlRpcServerMethod("OpenDB", s) {}


		void execute(XmlRpcValue& params, XmlRpcValue& result)
		{
			//to change dicomdir all views has to be closed
			if (GNC::GCS::ControladorVistas::Instance()->GetVistas().size() != 0) {
				result["error"] = -1;
				return;
			}

			std::string stdPath = std::string(params[0]["path"]);
			wxString newPath = FROMPATH(stdPath);

			if(!wxDir::Exists(newPath)) {
				//if the directory doesn't exists we create a new
	#ifdef _WIN32
				wxMkdir(newPath,511);
	#else
				wxMkDir(newPath.ToUTF8(), 0770);
	#endif
				if(!wxDirExists(newPath))
				{
					result["error"] = -1;
					return;
				}
			} else {
				//check if directory is readonly, to check it we write and erase a file
				wxString pathFicheroPrueba = newPath + wxFileName::GetPathSeparator() + wxString::Format(wxT("%d"),rand());
				while(wxFileExists(pathFicheroPrueba)) {
					pathFicheroPrueba = newPath + wxFileName::GetPathSeparator() + wxString::Format(wxT("%d"),rand());
				}
				{
					wxFile ficheroTmp;
					if(!ficheroTmp.Create(pathFicheroPrueba,false)){
						if(!wxFileExists(pathFicheroPrueba) ) {
							result["error"] = -1;
							return;
						}
					}
					ficheroTmp.Write(wxT("prueba"));
					ficheroTmp.Close();
				}
				wxRemoveFile(pathFicheroPrueba);
			}

			wxConfigBase * config = wxConfigBase::Get();
			//update path stored in the ini file
			config->SetPath(wxT("/GinkgoCore/Estacion"));

			wxString oldPath;
			config->Read(wxT("DicomDir"), &oldPath, wxEmptyString);

			wxFileName dirAnterior(oldPath);
			wxFileName dirNuevo(newPath);
			if(dirAnterior != dirNuevo)
			{
				config->Write(wxT("DicomDir"),newPath);
				//command is thrown to make update
				GNC::GCS::ControladorComandos::Instance()->ProcessAsync(_Std("Closing windows..."), new ChangeDicomDirCommand(), NULL);
			}
			result["error"] = 0;
		}

		std::string help() { return std::string("Changes Ginkgo database"); }

	};
	//---------------------------------------------------------------------------------------------------
	// "GetDisplayed2DViewerSeries" resturns series instance uid of opened series
	class GetDisplayed2DViewerSeries : public XmlRpcServerMethod
	{
	public:
		GetDisplayed2DViewerSeries(XmlRpcServer* s) : XmlRpcServerMethod("GetDisplayed2DViewerSeries", s) {}

		void execute(XmlRpcValue& /*params*/, XmlRpcValue& result)
		{
			//historial knows opened series
			GNC::GUI::PanelHistorial2* pPanel = GNC::Entorno::Instance()->GetVentanaPrincipal()->GetPanelHistorial();
			std::list<std::string> openedUIDs = pPanel->GetOpenedSeriesUIDs();
			int i = 0;
			for (std::list<std::string>::iterator it = openedUIDs.begin(); it != openedUIDs.end(); ++it)
			{
				result["elements"][i++]["seriesInstanceUID"] = (*it);
			}
			result["error"] = 0;
		}

		std::string help() { return std::string("Returns displayed 2D viewer series"); }

	};
	//---------------------------------------------------------------------------------------------------
	// "GetDisplayed2DViewerSeries" resturns study instance uid of opened series
	class GetDisplayed2DViewerStudies : public XmlRpcServerMethod
	{
	public:
		GetDisplayed2DViewerStudies(XmlRpcServer* s) : XmlRpcServerMethod("GetDisplayed2DViewerStudies", s) {}

		void execute(XmlRpcValue& /*params*/, XmlRpcValue& result)
		{
			//historial knows opened series
			GNC::GUI::PanelHistorial2* pPanel = GNC::Entorno::Instance()->GetVentanaPrincipal()->GetPanelHistorial();
			std::list<std::string> openedUIDs = pPanel->GetOpenedStudiesUIDs();
			int i = 0;
			for (std::list<std::string>::iterator it = openedUIDs.begin(); it != openedUIDs.end(); ++it)
			{
				result["elements"][i++]["studyInstanceUID"] = (*it);
			}
			result["error"] = 0;
		}

		std::string help() { return std::string("Returns displayed 2D viewer studies"); }

	};
	//---------------------------------------------------------------------------------------------------
	// CMove launches a ComandoPACS command to retrieve series or studies
	class CMove : public XmlRpcServerMethod
	{
	public:
		CMove(XmlRpcServer* s) : XmlRpcServerMethod("CMove", s) {}

		void execute(XmlRpcValue& params, XmlRpcValue& result)
		{
			std::string serverId = std::string(params[0]["server"]);
			std::string accesssionNumber = std::string(params[0]["accessionNumber"]);
			std::string studyInstanceUID = std::string(params[0]["studyInstanceUID"]);
			std::string seriesInstanceUID = std::string(params[0]["seriesInstanceUID"]);

			if (seriesInstanceUID != "") {
				GADAPI::ComandoPACSParams * pParams = new GADAPI::ComandoPACSParams(serverId, seriesInstanceUID, GADAPI::ComandoPACSParams::TA_Serie);
				GADAPI::ComandoPACS* pComandoPACS = new GADAPI::ComandoPACS(pParams);
				GNC::Entorno::Instance()->GetControladorComandos()->ProcessAsync(_Std("Downloading from PACS..."), pComandoPACS, NULL);
			} else if (accesssionNumber != "" || studyInstanceUID != "")
			{
				GIL::DICOM::TipoJerarquia base;
				base.tags["0008|0052"] = "STUDY";
				if (studyInstanceUID != "") {
					base.tags["0020|000d"] = studyInstanceUID;					
				}
				if (accesssionNumber != "") {
					base.tags["0008|0050"] = accesssionNumber;					
				}

				GADAPI::ComandoPACSParams * pParams = new GADAPI::ComandoPACSParams("", base);
				GADAPI::ComandoPACS* pComandoPACS = new GADAPI::ComandoPACS(pParams);
				GNC::Entorno::Instance()->GetControladorComandos()->ProcessAsync("Descargando PACS...",pComandoPACS, NULL);
			}
			result["error"] = 0;
		}

		std::string help() { return std::string("Retrieve files from PACS"); }
	};
	//---------------------------------------------------------------------------------------------------
	// Dicomize starts a dicomization workflow
	//LaunchDicomizationCommand opens importation wizard
	class LaunchDicomizationCommand : public GNC::GCS::IComando {
	public:
		LaunchDicomizationCommand(const std::string& uidModule, GnkPtr<GIL::IModeloIntegracion>  pModelo) : IComando(NULL)
		{
			SetId(IDC_RPC);
			m_uidModulo = uidModule;
			m_pModelo = pModelo;
		}

		virtual void Execute() {}

		virtual void Update()
		{
			try{
				GNC::GUI::wxWizardImportacionGinkgo ib(NULL, m_pModelo, m_uidModulo);
				ib.ShowModal();
			}
			catch(GIL::DICOM::I2DException& i){
				LOG_ERROR("GIL/WF/Importar", (const std::string)i);
			}
			catch(std::exception& ex1){
				LOG_ERROR("GIL/WF/Importar", std::string("Error al importar: ") + ex1.what());
			}
			catch (...)
			{
				LOG_ERROR("GIL/WF/Importar", std::string("Error al importar: Error interno"));
			}
		}
		GnkPtr<GIL::IModeloIntegracion> m_pModelo;
		std::string m_uidModulo;
	};
	// Dicomize parses params and launches command if necessary, parsing is similar to done in controladorhl7.cpp
	class Dicomize : public XmlRpcServerMethod
	{
	public:
		Dicomize(XmlRpcServer* s) : XmlRpcServerMethod("Dicomize", s) {}

		void execute(XmlRpcValue& params, XmlRpcValue& result)
		{
			GnkPtr<GIL::IModeloIntegracion> pModelo(new GIL::IModeloIntegracion());
			//general info
			pModelo->PACSAlmacenamiento = std::string(params[0]["pacsStoreSid"]);
			pModelo->idPlantilla = std::string(params[0]["Pid"]);
			pModelo->idPeticion = std::string(params[0]["PetitionId"]);
			//patient info
			pModelo->Paciente.nombre = std::string(params[0]["Patient"]["Name"]);
			pModelo->Paciente.apellido1 = std::string(params[0]["Patient"]["FirstSurname"]);
			pModelo->Paciente.apellido2 = std::string(params[0]["Patient"]["SecondSurname"]);
			pModelo->Paciente.numEpisodio = std::string(params[0]["Patient"]["EpisodeNumber"]);
			pModelo->Paciente.listaIdentificadores.push_back(GIL::IModeloIdentificador(std::string(params[0]["Patient"]["IdCode"]), std::string(params[0]["Patient"]["IdValue"])));
			//physician info
			pModelo->Medico.nombre = std::string(params[0]["Physician"]["Name"]);
			pModelo->Medico.apellido1 = std::string(params[0]["Physician"]["FirstSurname"]);
			pModelo->Medico.apellido2 = std::string(params[0]["Physician"]["SecondSurname"]);
			pModelo->Medico.idCentro = std::string(params[0]["Physician"]["IdCenter"]);
			pModelo->Medico.nombreCentro = std::string(params[0]["Physician"]["CenterName"]);
			pModelo->Medico.listaIdentificadores.push_back(GIL::IModeloIdentificador(std::string(params[0]["Physician"]["IdCode"]), std::string(params[0]["Physician"]["IdValue"])));
			//HCE
			pModelo->HCE.Sid = std::string(params[0]["HCE"]["AID"]);

			if(pModelo->HCE.Sid == "") {
				LOG_ERROR(LOGGER,_Std("id_application expected"));
			}
			if(pModelo->Medico.listaIdentificadores.size() == 0) {
				LOG_ERROR(LOGGER,_Std("It has been read a \"medico\" tag without identifiers"));
			}
			if(pModelo->Paciente.listaIdentificadores.size() == 0 && pModelo->accion == GIL::IModeloIntegracion::TA_Dicomizar) {
				LOG_ERROR(LOGGER,_Std("It has been read a \"paciente\" tag without identifiers"));
			}
			std::string codigoAplicacion;
			//se extrae de la configuracion el codigo de aplicacion
			wxString valor;
			wxConfigBase* config = wxConfigBase::Get();
			config->SetPath(wxT("/GinkgoCore/HCE"));

			config->Read(wxT("CodigoAplicacion"), &valor, wxEmptyString);
			pModelo->CodigoAplicacion = valor.ToUTF8();

			try {
				GIL::HL7::ControladorHL7 controlador;
				std::string uidModulo = controlador.GetUID(pModelo->idPlantilla);
				pModelo->UIDModulo = uidModulo;
				if (GNC::ControladorExtensiones::Instance()->ObtenerModulo(uidModulo) == NULL) {
					LOG_ERROR(LOGGER,"No existe el modulo asociado");
					result["error"] = -1;
					return;
				}
				GNC::GCS::ControladorComandos::Instance()->ProcessAsync(_Std("Closing windows..."), new LaunchDicomizationCommand(uidModulo,pModelo), NULL);
			}catch(GIL::DICOM::I2DException& i){
				LOG_ERROR("GIL/WF/Importar", (const std::string)i);
				result["error"] = -1;
				return;
			}
			catch(std::exception& ex1){
				LOG_ERROR("GIL/WF/Importar", std::string("Error al importar: ") + ex1.what());
				result["error"] = -1;
				return;
			}
			catch (...)
			{
				LOG_ERROR("GIL/WF/Importar", std::string("Error al importar: Error interno"));
				result["error"] = -1;
				return;
			}


			result["error"] = 0;
		}

		std::string help() { return std::string("Retrieve files from PACS"); }
	};
	//---------------------------------------------------------------------------------------------------
	// "DBWindowFind" executes actions over series, images or studies
	// LaunchOpenDeleteSeriesCommand it's necessary to open and delete series or studies from the history
	class LaunchOpenDeleteSeriesCommand : public GNC::GCS::IComando {
	public:
		LaunchOpenDeleteSeriesCommand(const std::list<std::string>& uidsSerie, bool open) : IComando(NULL)
		{
			SetId(IDC_RPC);
			m_uidsSerie = uidsSerie;
			m_open = open;
		}

		virtual void Execute() {}

		virtual void Update()
		{
			if (m_open) {
				GNC::GCS::ControladorEventos::Instance()->ProcesarEvento(new GNC::GCS::Eventos::EvtHandleDicom(m_uidsSerie, GNC::GCS::Eventos::EvtHandleDicom::OpenSeries));
			} else {
				GNC::GCS::ControladorEventos::Instance()->ProcesarEvento(new GNC::GCS::Eventos::EvtHandleDicom(m_uidsSerie, GNC::GCS::Eventos::EvtHandleDicom::DeleteSeries));
			}
		}
		std::list<std::string> m_uidsSerie;
		bool m_open;
	};

	//DBWindowFind xml rpc method, parses params and throws command if necessary
	//it processes four actions Open->open series (from study or from image), Delete->deletes series, nothing->makes an sql request, select->not implemented
	//
	class DBWindowFind : public XmlRpcServerMethod
	{
	public:
		DBWindowFind(XmlRpcServer* s) : XmlRpcServerMethod("DBWindowFind", s) {}

		void execute(XmlRpcValue& params, XmlRpcValue& result)
		{
			//Open opens series
			std::string action = std::string(params[0]["execute"]);
			std::string table = std::string(params[0]["table"]);
			std::string request = std::string(params[0]["request"]);
			try {
				if (action == "Open" || action == "Delete") {
					std::list<std::string> seriesUidList;
					if (table == "Series" || table == "Study") {
						GNC::GCS::IControladorHistorial::ListaModelosSeries modelList;
						GNC::GCS::ControladorHistorial::Instance()->GetSeriesModelSql(request, modelList);
						for (GNC::GCS::IControladorHistorial::ListaModelosSeries::iterator it = modelList.begin(); it != modelList.end(); ++it)
						{
							seriesUidList.push_back((*it).m_uidSerie);
						}
					} else if (table == "Image") {
						if (action == "Open") {
							GNC::GCS::IControladorHistorial::ListaModelosDCM modelList;
							std::set<std::string> uidSet;
							GNC::GCS::ControladorHistorial::Instance()->GetDCMModelSql(request, modelList);
							for (GNC::GCS::IControladorHistorial::ListaModelosDCM::iterator it = modelList.begin(); it != modelList.end(); ++it)
							{
								if (uidSet.find((*it).m_uidSerie) == uidSet.end()) {
									seriesUidList.push_back((*it).m_uidSerie);
									uidSet.insert((*it).m_uidSerie);
								}
							}
						} else {
							result["error"] = -1; //not implemented
							return;
						}
					}
					if (action == "Open") {
						GNC::GCS::ControladorComandos::Instance()->ProcessAsync(_Std("Opening series..."), new LaunchOpenDeleteSeriesCommand(seriesUidList,true), NULL);
					} else {
						GNC::GCS::ControladorComandos::Instance()->ProcessAsync(_Std("Deleting series..."), new LaunchOpenDeleteSeriesCommand(seriesUidList,false), NULL);
					}
					result["error"] = 0;
				} else if (action == "Nothing") {
					if (table == "Study") {
						GNC::GCS::IControladorHistorial::ListaModelosEstudios modelList;
						GNC::GCS::ControladorHistorial::Instance()->GetStudyModelSql(request, modelList);
						int i = 0;
						for (GNC::GCS::IControladorHistorial::ListaModelosEstudios::iterator it = modelList.begin(); it != modelList.end(); ++it) {
							result["elements"][i]["studyInstanceUID"] = (*it).m_uidEstudio;
							result["elements"][i]["StudyDescription"] = (*it).m_descripcionEstudio;
							result["elements"][i]["StudyDate"] = (*it).m_fechaEstudio;
							result["elements"][i++]["StudyTime"] = (*it).m_horaEstudio;
						}

					} else if (table == "Series") {
						GNC::GCS::IControladorHistorial::ListaModelosSeries modelList;
						GNC::GCS::ControladorHistorial::Instance()->GetSeriesModelSql(request, modelList);
						int i = 0;
						for (GNC::GCS::IControladorHistorial::ListaModelosSeries::iterator it = modelList.begin(); it != modelList.end(); ++it) {
							//patient info
							result["elements"][i]["PatientID"] = (*it).m_idPaciente;
							result["elements"][i]["PatientName"] = (*it).m_nombrePaciente;
							result["elements"][i]["PatientSex"] = (*it).m_sexo;
							//study info
							result["elements"][i]["studyInstanceUID"] = (*it).m_uidEstudio;
							result["elements"][i]["StudyDescription"] = (*it).m_descripcionEstudio;
							result["elements"][i]["StudyDate"] = (*it).m_fechaEstudio;
							result["elements"][i]["StudyTime"] = (*it).m_horaEstudio;
							//series info
							result["elements"][i]["seriesInstanceUID"] = (*it).m_uidSerie;
							result["elements"][i]["SeriesDescription"] = (*it).m_descripcionSerie;
							result["elements"][i]["SeriesDate"] = (*it).m_fechaSerie;
							result["elements"][i]["SeriesTime"] = (*it).m_horaSerie;
							result["elements"][i++]["SeriesModality"] = (*it).m_modalidad;
						}
					} else if (table == "Image") {
						GNC::GCS::IControladorHistorial::ListaModelosDCM modelList;
						std::set<std::string> uidSet;
						GNC::GCS::ControladorHistorial::Instance()->GetDCMModelSql(request, modelList);
						int i = 0;
						for (GNC::GCS::IControladorHistorial::ListaModelosDCM::iterator it = modelList.begin(); it != modelList.end(); ++it) {
							//patient info
							result["elements"][i]["PatientID"] = (*it).m_idPaciente;
							result["elements"][i]["PatientName"] = (*it).m_nombrePaciente;
							result["elements"][i]["PatientSex"] = (*it).m_sexo;
							//study info
							result["elements"][i]["studyInstanceUID"] = (*it).m_uidEstudio;
							result["elements"][i]["StudyDescription"] = (*it).m_descripcionEstudio;
							result["elements"][i]["StudyDate"] = (*it).m_fechaEstudio;
							result["elements"][i]["StudyTime"] = (*it).m_horaEstudio;
							//series info
							result["elements"][i]["seriesInstanceUID"] = (*it).m_uidSerie;
							result["elements"][i]["SeriesDescription"] = (*it).m_descripcionSerie;
							result["elements"][i]["SeriesDate"] = (*it).m_fechaSerie;
							result["elements"][i]["SeriesTime"] = (*it).m_horaSerie;
							result["elements"][i++]["SeriesModality"] = (*it).m_modalidad;
						}
					}
				} else if (action == "Select") {
					//TODO, select in the medical history
				}
				result["error"] = 0;
			}
			catch (wxSQLite3Exception& ex1) {
				LOG_ERROR(LOGGER, "SQL error executing DBWindowFind " <<ex1.GetMessage().ToUTF8());
				result["error"] = "Wrong sql syntax";
			}
			catch (std::exception ex) {
				LOG_ERROR(LOGGER, "SQL error executing DBWindowFind " << ex.what());
				result["error"] = "Wrong sql syntax";
			}
		}

		std::string help() { return std::string("open, delete and look up series, studies, patient..."); }

	};
	//---------------------------------------------------------------------------------------------------
	//---------------------------------------------------------------------------------------------------
}
//


void GIL::XMLRPC::XMLRPCController::FreeInstance()
{
	wxCriticalSectionLocker locker(m_criticalSection);
	if(m_pInstance != NULL){
		LOG_DEBUG(LOGGER, "Stopping XML-RPC Server");
		m_pInstance->Abort();
		m_pInstance = NULL;
	}
}

void GIL::XMLRPC::XMLRPCController::StartServer()
{
	wxConfigBase * config = wxConfigBase::Get();
	//se actualiza el fichero de configuracion
	config->SetPath(wxT("/GinkgoCore/HCE"));
	bool boolValue;
	config->Read(wxT("XMLRPCServerEnabled"),&boolValue,false);
	if (boolValue) {
		wxCriticalSectionLocker locker(m_criticalSection);		
		if (m_pInstance == NULL) {
			m_pInstance = new GIL::XMLRPC::XMLRPCController();
			m_pInstance->Create();
			GNC::GCS::Threading::SetThreadName( m_pInstance->GetId(), "XML-RPCServer");
			wxThreadError error = m_pInstance->Run();
			if(error != wxTHREAD_NO_ERROR) {
				LOG_ERROR(LOGGER, "Error running XML-RPC Server")
			} else {
				LOG_INFO(LOGGER, "Running XML-RPC Server")
			}
		} else {
			m_pInstance->Abort();
			m_pInstance = new GIL::XMLRPC::XMLRPCController();
			m_pInstance->Create();
			GNC::GCS::Threading::SetThreadName( m_pInstance->GetId(), "XML-RPCServer");
			wxThreadError error = m_pInstance->Run();
			if(error != wxTHREAD_NO_ERROR) {
				LOG_ERROR(LOGGER, "Error running XML-RPC Server")
			} else {
				LOG_INFO(LOGGER, "Running XML-RPC Server")
			}
		}
	}
}


GIL::XMLRPC::XMLRPCController* GIL::XMLRPC::XMLRPCController::m_pInstance = NULL;
wxCriticalSection GIL::XMLRPC::XMLRPCController::m_criticalSection;

//proceso que envia mensajes
GIL::XMLRPC::XMLRPCController::XMLRPCController() : wxThread(wxTHREAD_DETACHED)
{
	m_pServer = new XmlRpc::XmlRpcServer();
	m_pLogHandler = new XmlRpc::GinkgoLogHandler();

}

GIL::XMLRPC::XMLRPCController::~XMLRPCController()
{
	m_pServer->shutdown(true);
	delete m_pServer;
	m_pServer = NULL;
	delete m_pLogHandler;
	m_pLogHandler = NULL;
}

void GIL::XMLRPC::XMLRPCController::Abort()
{
	m_pServer->exit();
	m_pServer->shutdown(true);
}

void* GIL::XMLRPC::XMLRPCController::Entry()
{
	
//TODO it doesn't work with ControladorLog, throws an exception in the client
//	XmlRpc::XmlRpcLogHandler::setLogHandler(m_pLogHandler);
	XmlRpc::XmlRpcLogHandler::setVerbosity(5);

	
	wxConfigBase * config = wxConfigBase::Get();
	//se actualiza el fichero de configuracion
	config->SetPath(wxT("/GinkgoCore/HCE"));
	wxString wxTmp;
	config->Read(wxT("XMLRPCServerPort"),&wxTmp,wxEmptyString);

	long port;
	if (wxTmp.ToLong(&port)) {
		//register methods
		XmlRpc::CloseAllWindows closeAll(m_pServer);
		XmlRpc::OpenDB openDB(m_pServer);
		XmlRpc::GetDisplayed2DViewerSeries displayedSeries(m_pServer);
		XmlRpc::GetDisplayed2DViewerStudies displayedStudies(m_pServer);
		XmlRpc::Close2DViewerWithSeriesUID closeSeries(m_pServer);
		XmlRpc::Close2DViewerWithStudyUID closeStudy(m_pServer);
		XmlRpc::CMove cMove(m_pServer);
		XmlRpc::Dicomize dicomize(m_pServer);
		XmlRpc::DBWindowFind find(m_pServer);

		m_pServer->bindAndListen(port);

		// Enable introspection
		m_pServer->enableIntrospection(true);

		m_pServer->work(-1);
		
	} else {
		LOG_ERROR(LOGGER, "XML-RPC port is not a valid integer")
	}
	wxCriticalSectionLocker locker(m_criticalSection);
	m_pInstance = NULL;

	return NULL;
}





