/*
*  
*  $Id: gwaveformview.cpp 3697 2011-04-14 12:23:47Z tovar $
*  Ginkgo CADx Project
*
*  Copyright 2008-10 MetaEmotion S.L. All rights reserved.
*  http://ginkgo-cadx.com
*
*  This file is licensed under LGPL v3 license.
*  See License.txt for details
*
*
*/
#include "gwaveformview.h"
#include <wx/confbase.h>
#include <eventos/eventosginkgo.h>
#include <wx/pen.h>
#include <wx/aui/auibar.h>
#include <wx/ginkgostyle/ginkgostyle.h>

#include <visualizator/vistas/waveformview.h>
#include <visualizator/estudios/visualizatorstudy.h>
#include <api/icontroladorcomandos.h>
#include <api/icontroladoreventos.h>
#include <api/iwidgetsrenderer.h>
#include <visualizator/widgets/wecg.h>
#include <vtk/widgets/widgetsactor.h>
#include <widgets/wanotacionesquina.h>
#include <export/iherramientaanotacionesesquina.h>


#include <recursos/aprimresourcemanager.h>


#define ANNOTATOR_TEXT_COLOR 0.1f, 0.1f, 0.1f, 1.0f

#define ID_BUTTON_METADATA 0
#define ID_BUTTON_CORNER_ANNOTATIONS 1

namespace GNKVisualizator 
{
	namespace GUI 
	{
		//esta clase sirve para las barras de herramientas de abajo
		class GWaveformViewButtonBar:public wxAuiToolBar{
		public:
			GWaveformViewButtonBar(wxWindow* pParent, const wxColor& color = wxColor(0x40, 0x40, 0x40)): wxAuiToolBar(pParent,wxID_ANY, wxDefaultPosition, wxDefaultSize, wxAUI_TB_DEFAULT_STYLE)
			{
				SetToolBitmapSize(wxSize(16,16));
				AUI_NAMESPACE wxAuiDefaultToolBarArt* pToolArt= new AUI_NAMESPACE wxAuiDefaultToolBarArt();
		#if defined(USE_PATCHED_LIBS) && !defined(__WXOSX__)
				pToolArt->SetBaseColor(color);
		#endif
				SetArtProvider(pToolArt);
			}

			~GWaveformViewButtonBar()
			{
			}
		};
		
		/* Constructor */
		GWaveformView::GWaveformView(GNKVisualizator::WaveFormView* pView) : GWaveformViewBase(pView->GetEstudio()->VentanaPadre)
		{
			Freeze();
			View = pView;
			WECG = NULL;

			ViewImage2D->SetBackgroundColor(0.0f,0.0f,0.0f);
			ViewInteractor2D->m_pImageViewer = ViewImage2D;
			View->GetEstudio()->SetViewer(ViewImage2D);
			ViewImage2D->SetInputConnection(View->GetEstudio()->renderConnection);
			ViewImage2D->SetShowAnnotations(false);
			ViewImage2D->SetInteractionStyle(vtkGinkgoImageViewer::ZOOM_INTERACTION);

			// Setup del manejador de widgets */
			GNC::GCS::IContratoWidgets::m_pManager = View->GetEstudio()->Entorno->NewWidgetsManager(View);
			GNC::GCS::IContratoWidgets::Setup(GNC::GCS::IContratoWidgets::m_pManager);
			ViewInteractor2D->SetManager(m_pManager);
			ViewInteractor2D->SetVID(0,false);
			GNC::GCS::IContratoWidgets::m_pManager->SetRendererActivo(ViewInteractor2D);

			GNC::GCS::IContratoWidgets::m_pManager->CrearWidgetAnotador(this);

			WidgetRepresentation* pWidgetsRepresentation = WidgetRepresentation::New();
			pWidgetsRepresentation->SetRenderer(ViewInteractor2D->FindPokedRenderer(0,0));
			ViewInteractor2D->FindPokedRenderer(0,0)->AddViewProp(pWidgetsRepresentation);
			ViewInteractor2D->SetRepresentation(pWidgetsRepresentation);
			pWidgetsRepresentation->SetWidgetsManager(GNC::GCS::IContratoWidgets::m_pManager);
			pWidgetsRepresentation->SetWidgetsRenderer(ViewInteractor2D);
			pWidgetsRepresentation->Delete();
			
			// Setup de los contratos
			GNC::GCS::IContratoWidgets::Setup(GNC::GCS::IContratoWidgets::m_pManager);

			//events
			GNC::GCS::Eventos::EventoModificacionImagen ev2(View);
			GNC::GCS::Eventos::EventoModificacionWidget ev3(View);
			GNC::GCS::Eventos::EventoRender ev4(View);

			m_pButtonBar = new GWaveformViewButtonBar(m_pManipulationPanel);
			m_pButtonBar->AddTool(ID_BUTTON_METADATA,_("Show/Hide metadata"),APrimResourcesManager::Herramientas::GetIcoECGMetadata(),_("Show/Hide metadata"));
			m_pButtonBar->Connect(ID_BUTTON_METADATA,wxEVT_COMMAND_TOOL_CLICKED, wxCommandEventHandler( GWaveformView::OnShowHideMetadata),NULL,this);

			GNC::GCS::IControladorHerramientas* cH = View->GetEstudio()->Entorno->GetControladorHerramientas();
			GNC::GCS::IHerramientaAnotacionesEsquina*  hAnotacionesEsquina = cH->ObtenerHerramientaConcreta<GNC::GCS::IHerramientaAnotacionesEsquina>(GNC::GCS::IHerramientaAnotacionesEsquina::ID);

			m_pButtonBar->AddTool(ID_BUTTON_CORNER_ANNOTATIONS,_("Show/Hide corner annotations"),hAnotacionesEsquina->GetIcono() ,_("Show/Hide corner annotations"));
			m_pButtonBar->Connect(ID_BUTTON_CORNER_ANNOTATIONS,wxEVT_COMMAND_TOOL_CLICKED, wxCommandEventHandler( GWaveformView::OnShowHideCornerAnnotations),NULL,this);

			m_pButtonBar->Realize();
			m_pSizerRight->Add(m_pButtonBar,0, wxALIGN_CENTER_VERTICAL|wxEXPAND,0);

			
			
			View->GetEstudio()->Entorno->GetControladorEventos()->Registrar(this, ev2);
			View->GetEstudio()->Entorno->GetControladorEventos()->Registrar(this, ev3);
			View->GetEstudio()->Entorno->GetControladorEventos()->Registrar(this, ev4);

			
			Layout();
			Thaw();
		}


		GWaveformView::~GWaveformView()
		{
			if (View != NULL) {
				View->GetEstudio()->Entorno->GetControladorComandos()->AbortarComandosDeOwner(View);
			}
			GNC::GCS::IContratoWidgets::m_pManager->EliminarTodosLosWidgets(false);
			
			ViewInteractor2D->SetRepresentation(NULL);

			View->GetEstudio()->Entorno->FreeWidgetsManager(GNC::GCS::IContratoWidgets::m_pManager);
			GNC::GCS::IContratoWidgets::m_pManager = NULL;
			
			ViewImage2D->Detach();
			ViewImage2D->Delete();
			ViewImage2D = NULL;

			ViewInteractor2D->Delete();
			ViewInteractor2D = NULL;

			View->Lock(GLOC());
			delete View;
		}

		void GWaveformView::InitPipeline()
		{
			ViewImage2D->SetInputConnection(View->GetEstudio()->renderConnection);

			m_pComboGroups->Clear();

			std::list<std::string> listOfLabels = View->GetECGStudy()->GetLabelOfMultiplexGroups();
			for (std::list<std::string>::iterator it = listOfLabels.begin(); it != listOfLabels.end(); ++it) {
				m_pComboGroups->AppendString(wxString::FromUTF8((*it).c_str()));
			}
			m_pComboGroups->Select(0);

			if(ViewImage2D != NULL) {
				// FIXME: Comprobar si el pipeline esta conectado.
				ViewImage2D->SetupPipeline();

				
				WECG = new GNKVisualizator::Widgets::WECG(GetManager(), 0);
				GetManager()->InsertarWidget(WECG, false);
				
				WECG->Reload(View->GetECGStudy()->GetListOfChannels());
				WECG->AutoCallibrate();

				m_pTextDiagnosis->SetValue(wxString::FromUTF8(View->GetECGStudy()->GetDiagnosis().c_str()));
				m_pTextMeasurements->SetValue(wxString::FromUTF8(View->GetECGStudy()->GetGlobalMeasurement().c_str()));

				//change color to widget anotacionesesquina
				for( GNC::GCS::ListaWidgets::iterator it = m_pManager->GetListaWidgets().begin(); it != m_pManager->GetListaWidgets().end(); it++) {
					GNC::GCS::Widgets::IWidget* w = *it;
					if (w->GetVID() == -1) {
						GNC::GCS::Widgets::WAnotador* wac = dynamic_cast<GNC::GCS::Widgets::WAnotador*> (w);
						if (wac != NULL) {
							wac->SetTextColor(GNC::GCS::GLHelper::TColor(ANNOTATOR_TEXT_COLOR));
						}
					}
				}

			}			
		}

		
		void GWaveformView::StopPipeline()
		{
			m_pManager->OcultarTodosLosWidgets(true, 666);
			m_pManager->DisableAnimation();
			m_pManager->Render();
		}

		void GWaveformView::OnFinishLoad()
		{
			m_pManager->OcultarTodosLosWidgets(true, 666);
			m_pManager->DisableAnimation();
			m_pManager->Render();
		}

		vtkGinkgoImageViewer* GWaveformView::GetViewer()
		{
			return ViewImage2D;
		}

		void GWaveformView::OnPaint(wxPaintEvent& event)
		{
			//std::cout << "GVistaCompleja::OnPaint()" << std::endl;
			GWaveformViewBase::OnPaint(event);
			if(View->GetEstudio()->Entorno->GetControladorVistas()->GetVistaActiva() == View)
			{
				wxPaintDC dc(this);
				wxColour colorLineaSeleccion(219, 219, 0, 255);
				dc.SetBrush(wxBrush(colorLineaSeleccion,wxTRANSPARENT));
				dc.SetPen(wxPen(colorLineaSeleccion, 3, wxSOLID));
				dc.DrawRectangle(wxRect(wxPoint(1, 1), wxPoint(dc.GetSize().x-1, dc.GetSize().y-1)));
			}
		}
	
		void GWaveformView::OnComboGroup(wxCommandEvent &event)
		{
			View->GetECGStudy()->SetIndexOfActualMultiplexGroup(m_pComboGroups->GetSelection());
			WECG->Reload(View->GetECGStudy()->GetListOfChannels());
			WECG->AutoCallibrate();

			m_pTextDiagnosis->SetValue(wxString::FromUTF8(View->GetECGStudy()->GetDiagnosis().c_str()));
			m_pTextMeasurements->SetValue(wxString::FromUTF8(View->GetECGStudy()->GetGlobalMeasurement().c_str()));
		}

		void GWaveformView::OnShowHideMetadata(wxCommandEvent &)
		{
			Freeze();
			if (!m_pPanelMetadata->IsShown()) {
				//show
				m_pSplitterPanel->SplitHorizontally(m_pPanelView, m_pPanelMetadata, -100);
			} else {
				//hide
				m_pSplitterPanel->Unsplit(m_pPanelMetadata);
			}
			Thaw();
		}

		void GWaveformView::OnShowHideCornerAnnotations(wxCommandEvent &event)
		{
			m_pManager->MostrarOcultarAnotador();			
			View->GetEstudio()->Entorno->GetControladorEventos()->ProcesarEvento(new GNC::GCS::Eventos::EventoRender(View));
		}


		//IAnotador
		std::string GWaveformView::GetDICOMTag(std::string tagId)
		{
			std::string str("");
			View->GetEstudio()->GetTagDiagnosticoActivo(tagId,str);

			return str;
		}

		std::string GWaveformView::GetTopLeftAnnotation(GNC::GCS::Contexto3D* c)
		{
			if (!this->ViewImage2D->IsInstalledAndInitialized() || View == NULL)
				return "";

			std::ostringstream os;
			std::string tag;

			tag = "0010|0010"; // Patient's Name

			os << _Std("Patient: ") << GetDICOMTag(tag).c_str() << std::endl;

			tag = "0010|0020"; // Patient's ID

			os << _Std("Patient Id: ") << GetDICOMTag(tag).c_str();

			return os.str();
		}

		std::string GWaveformView::GetTopRightAnnotation(GNC::GCS::Contexto3D* c)
		{
			if (!this->ViewImage2D->IsInstalledAndInitialized() || View == NULL)
				return "";

			std::ostringstream os;
			std::string tag;
			std::string str;

			tag = "0010|0040"; // Patient's Sex

			os << GetDICOMTag(tag).c_str() << " ";

			tag = "0010|0030"; // Patient's BirthDate
			str = GetDICOMTag(tag);
			if (str.size() == 8) {
				os << str.substr(6, 2).c_str() << "-";
				os << str.substr(4, 2).c_str() << "-";
				os << str.substr(0, 4).c_str() << " ";
			}

			tag = "0010|1010"; // Patient's Age
			str = GetDICOMTag(tag);
			if(str.size() > 0) {
				os << str.c_str() << "Y" << std::endl;
			}

			tag = "0008|0060"; // Modalidad
			os << _Std("Modality: ") << GetDICOMTag(tag).c_str() << std::endl;

			tag = "0008|0021"; // Series Date
			str = GetDICOMTag(tag);
			if (str.size() == 8) {
				os << str.substr(6, 2).c_str() << "-";
				os << str.substr(4, 2).c_str() << "-";
				os << str.substr(0, 4).c_str() << " ";
			}
			os<<std::endl;

			tag = "0008|103e"; // Series Description
			str = GetDICOMTag(tag);
			if (!str.empty()) {
				if(str.size()<18){
					os << str.c_str();
				}else {
					os << str.substr(0,18).c_str()<<"...";
				}
			}

			return os.str();
		}

		std::string GWaveformView::GetBottomLeftAnnotation(GNC::GCS::Contexto3D* c)
		{
			
			if (!this->ViewImage2D->IsInstalledAndInitialized() || View == NULL)
				return "";

			std::ostringstream os;
			std::string tag;
			std::string str;

			tag = "0008|0020"; // study Date
			str = GetDICOMTag(tag);
			if (str.size() == 8) {
				os << str.substr(6, 2).c_str() << "-";
				os << str.substr(4, 2).c_str() << "-";
				os << str.substr(0, 4).c_str() << " ";
			}
			os<<std::endl;

			tag = "0008|1030"; // Study Description
			str = GetDICOMTag(tag);
			if (!str.empty()) {
				if(str.size()<18){
					os << str.c_str()<<std::endl;
				}else {
					os << str.substr(0,18).c_str()<<"..."<<std::endl;
				}
			}

			tag = "0008|0080"; // Nombre del centro, si viene el centro en el estudio se pone si no el d los tags estaticos
			str = GetDICOMTag(tag);
			if (str.size() == 6) {
				os << str.substr(0, 2).c_str() << ":";
			} else {
				wxConfigBase* config = wxConfigBase::Get();
				config->SetPath(wxT("/GinkgoCore/Estacion"));

				wxString valor;
				config->Read(wxT("CentroNombre"),&valor,wxT(""));
				std::string centro(valor.ToUTF8());

				if (!centro.empty()) {
					os << centro.c_str() << std::endl;
				}
			}

			os << _Std("Made with Ginkgo CADx");
			return os.str();
		}

		std::string GWaveformView::GetBottomRightAnnotation(GNC::GCS::Contexto3D* c)
		{
			if (!this->ViewImage2D->IsInstalledAndInitialized() || View == NULL)
				return "";
			std::ostringstream os;
			os << "25 mm/s, 10 mm/mV";
			return os.str();
		}
		std::string GWaveformView::GetAnotacionPosicion(GNC::GCS::Vector* pPosicionRaton)
		{
			return "";
		}
		//end ianotador

		void GWaveformView::PreProcesarEvento(GNC::GCS::Eventos::IEvento* evt, GNC::GCS::IControladorEventos::TipoListaPunterosEventos& lista)
		{
			if (evt == NULL) {
				std::cerr << "Error: Evento nulo" << std::endl;
				return;
			}
			switch (evt->GetCodigoEvento()) {
				case ginkgoEVT_Core_ModificacionImagen:
					{
						GNC::GCS::Eventos::EventoModificacionImagen* pEvt = dynamic_cast<GNC::GCS::Eventos::EventoModificacionImagen*>(evt);
						if (pEvt == NULL) {
							std::cerr << "Error al interpretar evento como evento de modificacin de imagen: Evento = " << evt << std::endl;
							return;
						}
						switch (pEvt->GetTipo()) {
							case GNC::GCS::Eventos::EventoModificacionImagen::ImagenRecalibrada:
								{
									lista.push_back(new GNC::GCS::Eventos::EventoModificacionImagen(View,GNC::GCS::Eventos::EventoModificacionImagen::ImagenCargada, -1));
								}
								break;
							default:
								break;
						}
						if (pEvt->GetApilarRender()) {
							lista.push_back(new GNC::GCS::Eventos::EventoRender(View,pEvt->GetSliceAfectado()));
						}
					}
					break;
				default:
					break;
			}
		}

		void GWaveformView::ProcesarEvento(GNC::GCS::Eventos::IEvento *evt) 
		{
			switch (evt->GetCodigoEvento()) {
				case ginkgoEVT_Core_ModificacionImagen:
				{
					GNC::GCS::Eventos::EventoModificacionImagen* pEvt = dynamic_cast<GNC::GCS::Eventos::EventoModificacionImagen*>(evt);
					if (pEvt == NULL) {
						std::cerr << "Error al interpretar evento como evento de modificacin de imagen: Evento = " << evt << std::endl;
						return;
					}
					switch (pEvt->GetTipo()) {
						case GNC::GCS::Eventos::EventoModificacionImagen::ImagenModificada:
						case GNC::GCS::Eventos::EventoModificacionImagen::ImagenCargada:
						case GNC::GCS::Eventos::EventoModificacionImagen::SliceCambiado:
								{
									ViewImage2D->Modified();
									View->GetEstudio()->Entorno->GetControladorEventos()->ProcesarEvento(new GNC::GCS::Eventos::EventoModificacionImagen(View,GNC::GCS::Eventos::EventoModificacionImagen::AnotacionesEstaticasModificadas));
									WECG->Reload(View->GetECGStudy()->GetListOfChannels());
								}
								break;
						case GNC::GCS::Eventos::EventoModificacionImagen::ImagenRecalibrada:
							{
								View->GetEstudio()->RecalibrarImagenActiva(pEvt->GetNewSpacing(), pEvt->GetNewOrigin());
							}
						break;
						case GNC::GCS::Eventos::EventoModificacionImagen::VisualizacionImagenModificada:
							{
								//para que pille bien el window/level
								if(pEvt->ResetZoom())
								{
									ViewImage2D->ResetZoom();
								}
								View->GetEstudio()->Entorno->GetControladorEventos()->ProcesarEvento(new GNC::GCS::Eventos::EventoModificacionImagen(View,GNC::GCS::Eventos::EventoModificacionImagen::AnotacionesEstaticasModificadas));
							}
						break;
						case GNC::GCS::Eventos::EventoModificacionImagen::ImagenDescargada:
							{
							}
						break;
						
						case GNC::GCS::Eventos::EventoModificacionImagen::MapaModificado:
						case GNC::GCS::Eventos::EventoModificacionImagen::AnotacionesEstaticasModificadas:
						break;
					}
				}//fin modificacion
				break;

				case ginkgoEVT_Core_Render:
					{
						ViewInteractor2D->Refresh(false);
					}
					break;
				default:
					break;
			}			
		}
	}
}
