/*
 *
 *  $Id: surface.cpp 3558 2011-03-20 20:02:22Z carlos $
 *  Ginkgo CADx Project
 *
 *  Copyright 2008-10 MetaEmotion S.L. All rights reserved.
 *  http://ginkgo-cadx.com
 *
 *  This file is licensed under LGPL v3 license.
 *  See License.txt for details
 *
 *
 */

#include "surface.h"
#include <wx/dc.h>
#include <wx/aui/aui.h>
#include <wx/timer.h>
#include <wx/msgdlg.h>
#include <wx/filedlg.h>
#include <wx/filename.h>
#include <wx/dir.h>
#include <wx/config.h>

#include <api/math/geometria3d.h>
#include <api/ientorno.h>
#include <api/icontroladorcarga.h>
#include <main/controllers/controladorlog.h>
#include <main/controllers/controladorcomandos.h>
#include <main/gui/progress/statusbarprogreso.h>
#include <wx/ginkgostyle/ginkgostyle.h>

#include "volumedataset/volumedataset.h"
#include "pipelines/surfacepipeline.h"
#include "commands/voiextractioncommand.h"
#include "commands/surfacecommand.h"
#include <eventos/eventosginkgo.h>

#include "../../exportacion/tagsprivados.h"
#include <export/tagsprivadoscomunes.h>
#include "../../comandos/comandodicomizacionintegracion.h"
#include <api/internacionalizacion.h>

#include "../../../recursos/aprimresourcemanager.h"
#include <resources/ginkgoresourcemanager.h>

#define ID_HIDE_BAR 1

#define ID_CURSOR 0
#define ID_STEREO 1
#define ID_DEMO_MODE 2
#define ID_RESET 3
#define ID_SAVE 4

namespace MedicalViewer
{
	namespace Reconstruction
	{
		namespace GUI
		{
			class SurfaceReconstructionBar:public wxAuiToolBar{
			public:
				SurfaceReconstructionBar(wxWindow* pParent, wxSurfaceRendering* surfaceRenderingDialog): wxAuiToolBar(pParent,wxID_ANY, wxDefaultPosition, wxDefaultSize, wxAUI_TB_DEFAULT_STYLE)
				{
					m_pSurfaceRenderingDialog = surfaceRenderingDialog;

					SetToolBitmapSize(wxSize(16,16));
					AddTool( ID_CURSOR, _("Cursor"), APrimResourcesManager::Bar3D::GetIco3DArrow(),_("Cursor") );
					AddTool( ID_RESET, _("Reset Camera"), APrimResourcesManager::Herramientas::GetIcoReset(),_("Reset Camera") );
					AddSeparator();
					AddTool( ID_STEREO, _("Stereo view"), APrimResourcesManager::Bar3D::GetIcoStereo(),_("Stereo view"), wxITEM_CHECK );
					AddSeparator();
					AddTool( ID_DEMO_MODE, _("Demo mode"), APrimResourcesManager::Bar3D::GetIcoDemo(),_("Demo mode"), wxITEM_CHECK );
					AddSeparator();
					AddTool( ID_SAVE, _("Save Frame"), GinkgoResourcesManager::IconosMenus::GetIcoGuardar(),_("Save current frame to new serie") );
					Realize();
					Layout();


					// Connect Events
					this->Connect( ID_CURSOR, wxEVT_COMMAND_TOOL_CLICKED, wxCommandEventHandler( SurfaceReconstructionBar::OnBCursor ) );
					this->Connect( ID_STEREO, wxEVT_COMMAND_TOOL_CLICKED, wxCommandEventHandler( SurfaceReconstructionBar::OnBStereo ) );
					this->Connect( ID_DEMO_MODE, wxEVT_COMMAND_TOOL_CLICKED, wxCommandEventHandler( SurfaceReconstructionBar::OnBDemo ) );
					this->Connect( ID_SAVE, wxEVT_COMMAND_TOOL_CLICKED, wxCommandEventHandler( SurfaceReconstructionBar::OnBSave ) );
					this->Connect( ID_RESET, wxEVT_COMMAND_TOOL_CLICKED, wxCommandEventHandler( SurfaceReconstructionBar::OnBReset ) );

					Realize();
				}

				~SurfaceReconstructionBar()
				{
				}

				virtual void OnBCursor(wxCommandEvent&) {
				}

				virtual void OnBStereo(wxCommandEvent& event) {
					m_pSurfaceRenderingDialog->SetStereo(GetToolToggled(ID_STEREO));
					event.Skip(true);
				}

				virtual void OnBDemo(wxCommandEvent& event) {
					m_pSurfaceRenderingDialog->SetDemo(GetToolToggled(ID_DEMO_MODE));
					event.Skip(true);
				}

				virtual void OnBReset(wxCommandEvent&) {
					m_pSurfaceRenderingDialog->ResetCamera();
				}

				virtual void OnBSave(wxCommandEvent&) {
					m_pSurfaceRenderingDialog->SaveFrameToSerie();
				}


				wxSurfaceRendering* m_pSurfaceRenderingDialog;
			};

			class SurfaceTimerDemo : public wxTimer
			{
				typedef MedicalViewer::Reconstruction::Pipelines::SurfacePipeline TPipeline;
			public:
				SurfaceTimerDemo(GnkPtr<TPipeline>& pipeline):wxTimer()
				{
					Pipeline = pipeline;
				}

				~SurfaceTimerDemo()
				{}

				virtual void Notify()
				{
					Pipeline->RotateDemo(1);
					Pipeline->Render();
				}

				virtual void Start()
				{
					wxTimer::Start(100, false);
				}

				GnkPtr<TPipeline> Pipeline;
			};
		}
	}
}


MedicalViewer::Reconstruction::GUI::wxSurfaceRendering::wxSurfaceRendering(wxWindow* pParent, const GnkPtr<TStudy>& study) : wxSurfaceRenderingBase(pParent), RefStudy(GNC::GCS::IContextoEstudio::NewRef(study))
{

	typedef MedicalViewer::Reconstruction::Pipelines::SurfacePipeline              TDataSet;
	typedef MedicalViewer::Reconstruction::Commands::VOIExtractionCommandParams    TVOIParams;
	typedef MedicalViewer::Reconstruction::Commands::VOIExtractionCommand          TVOICommand;

	RefStudy->UnRefViewer();

	m_editing = false;
	m_pProgressBar = new GNC::GUI::StatusBarProgreso(this, this, false);
	SetStatusBar(m_pProgressBar);

	wxIcon icono;
	icono.CopyFromBitmap(GinkgoResourcesManager::Logos::GetLogoGinkgo32x32());
	this->SetIcon(icono);

	Pipeline = new TPipeline(ViewInteractor);
	m_pTimer = new MedicalViewer::Reconstruction::GUI::SurfaceTimerDemo(Pipeline);

	m_pPanelSurface2->Enable(false);

	m_pHeaderPanel->GetButtonBar()->AddTool(ID_HIDE_BAR,_("Hide"),APrimResourcesManager::Bar3D::GetIcoPin(),_("Hide"), wxITEM_NORMAL);
	m_pHeaderPanel->GetButtonBar()->Connect(ID_HIDE_BAR,wxEVT_COMMAND_TOOL_CLICKED, wxCommandEventHandler( MedicalViewer::Reconstruction::GUI::wxSurfaceRendering::OnToolTitleButtonClick),NULL,this);
	m_pHeaderPanel->Realize();

	//aui bar
	SurfaceReconstructionBar* m_pBar = new SurfaceReconstructionBar(this, this);
	m_pBarSizer->Add(m_pBar, 0, wxEXPAND);

	ViewInteractor->GetRenderWindow()->AddRenderer(Pipeline->GetRenderer());
	Pipeline->SetupInteractor();

	TVOICommand* voiCmd = new TVOICommand(new TVOIParams(RefStudy, Pipeline));
	GNC::GCS::ControladorComandos::Instance()->ProcessAsync("VOI Extraction", voiCmd, this);

	study->Entorno->GetControladorEventos()->Registrar(this, GNC::GCS::Eventos::EventoProgresoComando());

	m_pBody->Layout();
	m_pToolPanel->Layout();
	Layout();
}


MedicalViewer::Reconstruction::GUI::wxSurfaceRendering::~wxSurfaceRendering()
{
	GNC::GCS::ControladorComandos::Instance()->AbortarComandosDeOwnerAsincrono(this);
	ViewInteractor->Delete();
	ViewInteractor->Reparent(NULL);
	if (m_pTimer != NULL) {
		delete m_pTimer;
		m_pTimer = NULL;
	}
}

void MedicalViewer::Reconstruction::GUI::wxSurfaceRendering::SetDemo(bool enabled)
{
	if (enabled) {
		m_pTimer->Start();
	} else {
		m_pTimer->Stop();
	}
}

void MedicalViewer::Reconstruction::GUI::wxSurfaceRendering::ResetCamera()
{
	Pipeline->ResetCamera();
	Pipeline->Render();
}

void MedicalViewer::Reconstruction::GUI::wxSurfaceRendering::SaveFrameToSerie()
{
	wxString wxNuevoDirTemp;
	do {
		wxNuevoDirTemp = FROMPATH(RefStudy->Entorno->GetGinkgoTempDir()) + wxFileName::GetPathSeparator(wxPATH_NATIVE) + wxT("_gnktmp_") + wxString::Format(wxT("%d"), rand());
	} while(wxDir::Exists(wxNuevoDirTemp));

	#ifdef _WIN32
			wxFileName::Mkdir(wxNuevoDirTemp);
	#else
			wxFileName::Mkdir(wxNuevoDirTemp.c_str(), 0777);
	#endif

	wxString fileNameExport = wxNuevoDirTemp + wxFileName::GetPathSeparator() + wxT("1.jpg");
	Pipeline->Print(std::string(TOPATH(fileNameExport)));

	GnkPtr<GNKVisualizator::GUI::TipoWizardImportacion> pPersistentData(new GNKVisualizator::GUI::TipoWizardImportacion(NULL));

	GNKVisualizator::GUI::TipoWizardImportacion::TFicheroDicomizacionGNKVisualizator ficheroADicomizar;
	ficheroADicomizar.pathImagen = TOPATH(fileNameExport);
	pPersistentData->m_listaFicherosADicomizar.push_back(ficheroADicomizar);

	//institucion
	wxString wxTmp;
	wxConfigBase::Get()->Read(wxT("/GinkgoCore/Estacion/CentroNombre"),&wxTmp,wxEmptyString);
	if(wxTmp != wxEmptyString){
		pPersistentData->baseImagenes.tags[std::string("0008|0080")] = std::string(wxTmp.ToUTF8());
	}

	//nombre del medico responsable de la institucion

	wxConfigBase::Get()->Read(wxT("/GinkgoCore/Estacion/NombreMedico"),&wxTmp,wxEmptyString);
	if(wxTmp != wxEmptyString){
		pPersistentData->baseImagenes.tags[std::string("0008|0090")] = std::string(wxTmp.ToUTF8());
	}

	pPersistentData->baseImagenes.tags[std::string("0008|0070")] = std::string("Metaemotion S.L.");

	pPersistentData->baseImagenes.tags[std::string("0008|1090")] = std::string("Ginkgo Atencin Primaria");

	//uid de aprimaria
	pPersistentData->baseImagenes.tags[std::string("0018|1030")] = UID_APRIM;

	//study data
	//date
	std::string tmp;
	RefStudy->GetTagImagenActiva("0008|0020", tmp);
	pPersistentData->baseImagenes.tags[std::string("0008|0020")] = tmp;
	//time
	RefStudy->GetTagImagenActiva("0008|0030", tmp);
	pPersistentData->baseImagenes.tags[std::string("0008|0030")] = tmp;
	//description
	RefStudy->GetTagImagenActiva("0008|1030", tmp);
	pPersistentData->baseImagenes.tags[std::string("0008|1030")] = tmp;
	//uid
	RefStudy->GetTagImagenActiva("0020|000d", tmp);
	pPersistentData->baseImagenes.tags[std::string("0020|000d")] = tmp;
	pPersistentData->StudyInstanceUID = tmp;

	//serie and image data
	//date
	pPersistentData->baseImagenes.tags[std::string("0008|0022")] = pPersistentData->baseImagenes.tags[std::string("0008|0021")] = wxDateTime::Now().Format(wxT("%Y%m%d")).ToUTF8();
	//time
	pPersistentData->baseImagenes.tags[std::string("0008|0032")] = pPersistentData->baseImagenes.tags[std::string("0008|0031")] = wxDateTime::Now().Format(wxT("%H%M%S")).ToUTF8();
	//description
	pPersistentData->baseImagenes.tags[std::string("0020|4000")] = pPersistentData->baseImagenes.tags[std::string("0008|103e")] = _Std("3D Surface");

	//patient
	//name
	RefStudy->GetTagImagenActiva("0010|0010", tmp);
	pPersistentData->baseImagenes.tags[std::string("0010|0010")] = tmp;
	//id
	RefStudy->GetTagImagenActiva("0010|0020", tmp);
	pPersistentData->baseImagenes.tags[std::string("0010|0020")] = tmp;
	//age
	RefStudy->GetTagImagenActiva("0010|1010", tmp);
	pPersistentData->baseImagenes.tags[std::string("0010|1010")] = tmp;
	//birth date
	RefStudy->GetTagImagenActiva("0010|0030", tmp);
	pPersistentData->baseImagenes.tags[std::string("0010|0030")] = tmp;
	//sex
	RefStudy->GetTagImagenActiva("0010|0040", tmp);
	pPersistentData->baseImagenes.tags[std::string("0010|0040")] = tmp;

	GNKVisualizator::GADAPI::ComandoDicomizacionIntegracionParams* pDicomParams = new GNKVisualizator::GADAPI::ComandoDicomizacionIntegracionParams(pPersistentData,RefStudy->Entorno, std::string(TOPATH(wxNuevoDirTemp)));
	GNKVisualizator::GADAPI::ComandoDicomizacionIntegracion* pDicomCmd = new GNKVisualizator::GADAPI::ComandoDicomizacionIntegracion(pDicomParams);
	RefStudy->Entorno->GetControladorComandos()->ProcessAsync(_Std("Performing tasks of integration ..."),pDicomCmd,NULL);

	#if defined (_WINDOWS)
	wxRmDir(wxNuevoDirTemp);
	#else
	wxRmDir(std::string(wxNuevoDirTemp.ToUTF8()).c_str());
	#endif
}


void MedicalViewer::Reconstruction::GUI::wxSurfaceRendering::SetStereo(bool enabled)
{
	ViewInteractor->GetRenderWindow()->SetStereoTypeToRedBlue();
	ViewInteractor->GetRenderWindow()->SetStereoRender(enabled? 1: 0);
	ViewInteractor->Refresh(false);
}

void MedicalViewer::Reconstruction::GUI::wxSurfaceRendering::OnWindowSize( wxSizeEvent& /*event*/ )
{
	Freeze();
	Layout();
	Refresh(false);
	Thaw();
}

void MedicalViewer::Reconstruction::GUI::wxSurfaceRendering::OnToolTitleButtonClick( wxCommandEvent& event )
{
	Freeze();
	m_pToolPanel->Show(false);
	m_pToolTitlePanel->Show(true);
	Layout();
	Refresh(true);
	Thaw();
}

void MedicalViewer::Reconstruction::GUI::wxSurfaceRendering::OnTitleLeftUp( wxMouseEvent& /*event*/ )
{
	Freeze();
	m_pToolPanel->Show(true);
	m_pToolTitlePanel->Show(false);
	Layout();
	Refresh(true);
	Thaw();
}

void MedicalViewer::Reconstruction::GUI::wxSurfaceRendering::OnToolSurface1TextChanged( wxCommandEvent& /*event*/ )
{
	if (!m_editing) {
		m_pComboDefalut1->SetSelection(-1);
	}
}

void MedicalViewer::Reconstruction::GUI::wxSurfaceRendering::OnCheckEnable2( wxCommandEvent& /*event*/ )
{
	m_pPanelSurface2->Enable(m_pEnableSurface2->GetValue());
}

void MedicalViewer::Reconstruction::GUI::wxSurfaceRendering::OnToolSurface2TextChanged( wxCommandEvent& /*event*/ )
{
	if (!m_editing) {
		m_pComboDefault2->SetSelection(-1);
	}
}

void MedicalViewer::Reconstruction::GUI::wxSurfaceRendering::OnEraseBackground( wxEraseEvent& event )
{
	event.Skip(true);
}

void MedicalViewer::Reconstruction::GUI::wxSurfaceRendering::OnDataSetSliderChanged( wxScrollEvent& event )
{
}

void MedicalViewer::Reconstruction::GUI::wxSurfaceRendering::OnApplyClick(wxCommandEvent &event)
{
	typedef MedicalViewer::Reconstruction::Surface::Commands::SurfaceCommandParams TSurfaceParams;
	typedef MedicalViewer::Reconstruction::Surface::Commands::SurfaceCommand       TSurfaceCommand;

	long sval[2] = {0, 0};

	if (!m_pSurface1ValueEdit->GetValue().ToLong(&sval[0]))
	{
		wxMessageBox(_("Pixel value of first surface doesn't have a valid value"), _("Warning"),wxICON_WARNING);
		return;
	}

	if (m_pEnableSurface2->GetValue() && !m_pSurface2ValueEdit->GetValue().ToLong(&sval[1]))
	{
		wxMessageBox(_("Pixel value of second surface doesn't have a valid value"), _("Warning"),wxICON_WARNING);
		return;
	}

	Pipeline->Enable(0, false);
	Pipeline->Enable(1, false);
	Refresh(false);

	TSurfaceCommand* surfCmd = new TSurfaceCommand(
		new TSurfaceParams(
			Pipeline,                                             // Pipeline
			0,                                                    // DataSet
			m_pPreSmoothCheck->GetValue(),                        // PreSmooth
			m_pPostSmoothCheck->GetValue(),                       // PostSmooth
			m_pResolutionSlider->GetValue(),                      // Quality (0-3)
			TSurfaceParams::SurfaceProps(
				true,                                             // Surf 1: Enabled
				sval[0],                                          // Surf 1: ISOValue
				m_pSurface1CPicker->GetColour().Red() / 255.0f,   // Surf 1: Red
				m_pSurface1CPicker->GetColour().Green() / 255.0f, // Surf 1: Green
				m_pSurface1CPicker->GetColour().Blue() / 255.0f,  // Surf 1: Blue
				1.0                                               // Surf 1: Alpha
				),

			TSurfaceParams::SurfaceProps(
				m_pEnableSurface2->GetValue(),                    // Surf 2: Enabled
				sval[1],                                          // Surf 2: ISOValue
				m_pSurface2CPicker->GetColour().Red() / 255.0f,   // Surf 2: Red
				m_pSurface2CPicker->GetColour().Green() / 255.0f, // Surf 2: Green
				m_pSurface2CPicker->GetColour().Blue() / 255.0f,  // Surf 2: Blue
				(float) m_pSurface2Alpha->GetValue() / 100.0f     // Surf 2: Alpha
				)
			)
		);
	GNC::GCS::ControladorComandos::Instance()->ProcessAsync("Surface setup", surfCmd, this);
}


void MedicalViewer::Reconstruction::GUI::wxSurfaceRendering::OnComboFirstValue(wxCommandEvent &event)
{
	m_editing = true;
	SetValueCombo(m_pComboDefalut1->GetSelection(), m_pSurface1ValueEdit);
	m_editing = false;
	event.Skip(true);
}

void MedicalViewer::Reconstruction::GUI::wxSurfaceRendering::OnComboSecondValue(wxCommandEvent &event)
{
	m_editing = true;
	SetValueCombo(m_pComboDefault2->GetSelection(), m_pSurface2ValueEdit);
	m_editing = false;
	event.Skip(true);
}

void MedicalViewer::Reconstruction::GUI::wxSurfaceRendering::SetValueCombo(int selection, wxTextCtrl* pText)
{
	switch (selection)
	{
	case 0:
		//bone
		pText->SetValue(wxT("400"));
		break;
	case 1:
		//muscle
		pText->SetValue(wxT("40"));
		break;
	case 2:
		//skin
		pText->SetValue(wxT("-120"));
		break;
	default:
		break;
	}
}

void MedicalViewer::Reconstruction::GUI::wxSurfaceRendering::ProcesarEvento(GNC::GCS::Eventos::IEvento *evt)
{
	GNC::GCS::Eventos::EventoProgresoComando* pEvt = dynamic_cast<GNC::GCS::Eventos::EventoProgresoComando*> (evt);

	if (pEvt == NULL  || pEvt->GetComando() == NULL || pEvt->GetComando()->GetOwner() != this) {
		return;
	}
	switch (pEvt->GetTipo()) {
		case GNC::GCS::Eventos::EventoProgresoComando::TEP_Iniciado:
			//
			m_pProgressBar->InsertarTarea(pEvt->GetComando()->GetId(), pEvt->GetTexto());
			break;
		case GNC::GCS::Eventos::EventoProgresoComando::TEP_Progreso:
			//
			m_pProgressBar->SetProgresoTarea(pEvt->GetComando()->GetId(), pEvt->GetProgresoNormalizado(), pEvt->GetTexto());
			break;
		case GNC::GCS::Eventos::EventoProgresoComando::TEP_Finalizado:
			//
			m_pProgressBar->EliminarTarea(pEvt->GetComando()->GetId());
			break;
		case GNC::GCS::Eventos::EventoProgresoComando::TEP_Unknown:
			break;
	}

}
